/**
 * This file was automatically generated by openapi-codegeneration.
 * DO NOT MODIFY IT BY HAND. Instead, modify the source OpenAPI file,
 * and run openapi-codegeneration to regenerate this file.
 */
/* eslint-disable @typescript-eslint/no-unused-vars */
/* eslint-disable @typescript-eslint/ban-ts-comment */
import fetch from "node-fetch";
import * as RequestValidation from './requestValidation.js';
import * as Types from './types.js';
/**
 * This error class should be used if an error occurs during validation of a request/response.
 * @category Errors
 */
export class ValidationError extends Error {
    errors;
    constructor(message, errors) {
        super(message);
        this.name = "ValidationError";
        this.errors = errors;
    }
}
/**
 * This error class should be used if an error occurs while fetching a request.
 * @category Errors
 */
export class FetchError extends Error {
    constructor(message) {
        super(message);
        this.name = "FetchError";
    }
}
/**
 * This error class should be used if the response of the server has a status
 * greater than or equal to 400. This error should contain the validated response.
 * @category Errors
 */
export class UnsuccessfulRequestError extends Error {
    response;
    constructor(message, response) {
        super(message);
        this.response = response;
        this.name = 'UnsuccessfulRequestError';
    }
}
/**
 * This error class should be used if the validation of an url fails.
 * @category Errors
 */
export class InvalidUrlError extends Error {
    constructor(message) {
        super(message);
        this.name = 'InvalidUrlError';
    }
}
/**
 * This function attempts to parse a Response as ResponseData.
 */
async function parseResponse(response) {
    let text = null;
    try {
        text = await response.text();
        const json = JSON.parse(text);
        return { status: response.status, body: json };
    }
    catch {
        return { status: response.status, body: text };
    }
}
/**
 * This function checks if a string is a valid http url.
 * @param string The string to be checked.
 * @returns True if the string is a valid http url.
 */
function isValidHttpUrl(string) {
    let url;
    try {
        url = new URL(string);
    }
    catch (_) {
        return false;
    }
    return url.protocol === 'http:' || url.protocol === 'https:';
}
/**
 * This function takes an url and a pattern for the requested endpoint and parses the contained path parameters.
 * @param url The url to be parsed.
 * @param endpoint Pattern of the requested endpoint.
 * @throws {InvalidUrlError} Thrown if the provided url does not end with the provided endpoint.
 * @returns An array containing all parsed path parameters in order of appearance.
 * @example
 * // returns ["username", "role_name"]
 * parsePathParameters("https://api.example.com/users/username/roles/role_name", "/users/{}/roles/{}")
 */
function parsePathParameters(url, endpoint) {
    const parameterRegex = '([a-zA-Z0-9-:]+)';
    const regex = new RegExp(endpoint.replaceAll('{}', parameterRegex) + '(?:.(?!\\\\))?$');
    const matches = url.match(regex);
    if (!matches)
        throw new InvalidUrlError('Url does not end with the provided endpoint');
    return matches.slice(1);
}
/**
 * This function validates a given http url and parses its path parameters.
 * @param url The url to be validated.
 * @param endpoint Pattern of the requested endpoint.
 * @throws {InvalidUrlError} Thrown if the validation of the provided url fails.
 * @returns An array containing all parsed path parameters in order of appearance (see {@link parsePathParameters}).
 * @example
 * // returns ["username", "role_name"]
 * validateUrl("https://api.example.com/users/username/roles/role_name", "/users/{}/roles/{}")
 */
function validateUrl(url, baseUrl, endpoint) {
    if (!isValidHttpUrl(url))
        throw new InvalidUrlError('Provided url is not a valid http url');
    if (!url.startsWith(baseUrl))
        throw new InvalidUrlError('Provided url does not start with the provided base url');
    const pathParameters = parsePathParameters(url, endpoint);
    let extendedBaseUrl = baseUrl + endpoint;
    pathParameters.forEach(pathParameter => {
        extendedBaseUrl = extendedBaseUrl.replace('{}', pathParameter);
    });
    if (url !== extendedBaseUrl)
        throw new InvalidUrlError('Provided url does not match extended base url');
    return pathParameters;
}
/**
 * This function appends a given endpoint to an url.
 * @param url The url to append the endpoint to.
 * @param endpoint The endpoint to be appended.
 * @example
 * // returns "https://api.example.com/devices/device_id/token"
 * appendToUrl("https://api.example.com/devices/device_id", "/token")
 * @example
 * // returns "https://api.example.com/devices/device_id/token"
 * appendToUrl("https://api.example.com/devices/device_id/", "/token")
 * @returns The url with the appended endpoint.
 */
function appendToUrl(url, endpoint) {
    return url.endsWith('/') ? `${url.slice(0, -1)}${endpoint}` : `${url}${endpoint}`;
}
/**
 * Client for the use of the Crosslab API
 * @category Client
 */
export class Client {
    baseUrl;
    serviceUrl;
    accessToken;
    fixedHeaders;
    fetch = async (url, init) => {
        let raw_response;
        try {
            if (url.toString().startsWith(this.baseUrl) ||
                url.toString().startsWith(this.serviceUrl)) {
                raw_response = await fetch(url, init);
            }
            else {
                raw_response = await fetch(appendToUrl(this.baseUrl, '/proxy?' + new URLSearchParams([['URL', url.toString()]]).toString()), init);
            }
        }
        catch (error) {
            if (error instanceof Error) {
                throw new FetchError(error.message);
            }
            else if (typeof error === 'string') {
                throw new FetchError(error);
            }
            else {
                throw new FetchError('Something went wrong while trying to fetch the request');
            }
        }
        return await parseResponse(raw_response);
    };
    /**
     * @category Constructors
     */
    constructor(baseUrl, options) {
        this.baseUrl = baseUrl.endsWith('/') ? baseUrl.slice(0, -1) : baseUrl;
        this.serviceUrl = options.serviceUrl
            ? options.serviceUrl.endsWith('/')
                ? options.serviceUrl.slice(0, -1)
                : options.serviceUrl
            : this.baseUrl;
        this.accessToken = options.accessToken ?? '';
        this.fixedHeaders = options.fixedHeaders ?? [];
    }
    /**
     * List experiments
     *
     * @param options.url
     * Url of the  to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The list of all experiments.
     */
    async listExperiments(options) {
        const url = appendToUrl(options?.url ?? this.baseUrl, "/experiments");
        if (!RequestValidation.validateListExperimentsInput())
            throw new ValidationError('Request validation failed!', RequestValidation.validateListExperimentsInput.errors);
        const authorization = `Bearer ${this.accessToken}`;
        const response = await this.fetch(url.replace(this.baseUrl, this.serviceUrl), {
            method: "GET",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...this.fixedHeaders,
                ...(options?.headers ?? [])
            ]
        });
        if (!RequestValidation.validateListExperimentsOutput(response))
            throw new ValidationError('Response validation failed!', RequestValidation.validateListExperimentsOutput.errors);
        if (Types.isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Create a new experiment
     *
     * @param experiment
     * The experiment to be created.
     * @param options.changedURL
     * An URL that will be called when the experiment status changes.
     * @param options.url
     * Url of the  to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * 201: The experiment was created. A JSON representation of the new experiment is returned.
     * 202: The experiment was created and will be set to running eventually. A JSON representation of the new experiment is returned.
     */
    async createExperiment(experiment, options) {
        const url = appendToUrl(options?.url ?? this.baseUrl, "/experiments");
        const body = experiment;
        const parameters = {
            changedURL: options?.changedURL,
        };
        const query = [];
        if (parameters["changedURL"])
            query.push(["changedURL", parameters["changedURL"].toString()]);
        if (!RequestValidation.validateCreateExperimentInput(parameters, body))
            throw new ValidationError('Request validation failed!', RequestValidation.validateCreateExperimentInput.errors);
        const authorization = `Bearer ${this.accessToken}`;
        const response = await this.fetch(url.replace(this.baseUrl, this.serviceUrl) + '?' + new URLSearchParams(query), {
            method: "POST",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...this.fixedHeaders,
                ...(options?.headers ?? [])
            ],
            body: JSON.stringify(body)
        });
        if (!RequestValidation.validateCreateExperimentOutput(response))
            throw new ValidationError('Response validation failed!', RequestValidation.validateCreateExperimentOutput.errors);
        if (Types.isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * View an experiment.
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON Representation of the experiment
     */
    async getExperiment(url, options) {
        const urlSuffix = '/experiments/{}'.split('{}').at(-1) ?? '';
        if (urlSuffix && !url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [experiment_id,] = validateUrl(url, this.baseUrl, '/experiments/{}');
        const parameters = {
            experiment_id: experiment_id,
        };
        if (!RequestValidation.validateGetExperimentInput(parameters))
            throw new ValidationError('Request validation failed!', RequestValidation.validateGetExperimentInput.errors);
        const authorization = `Bearer ${this.accessToken}`;
        const response = await this.fetch(url.replace(this.baseUrl, this.serviceUrl), {
            method: "GET",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...this.fixedHeaders,
                ...(options?.headers ?? [])
            ]
        });
        if (!RequestValidation.validateGetExperimentOutput(response))
            throw new ValidationError('Response validation failed!', RequestValidation.validateGetExperimentOutput.errors);
        if (Types.isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Update an existing experiment.
     *
     * @param url
     * Url of the resource to be accessed.
     * @param experimentUpdate
     * Update the experiment
     * @param options.changedURL
     * An URL that will be called when the experiment status changes.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * 200: The JSON Representation of the changed experiment
     * 202: The JSON Representation of the changed experiment, that will be set to running eventually
     */
    async updateExperiment(url, experimentUpdate, options) {
        const urlSuffix = '/experiments/{}'.split('{}').at(-1) ?? '';
        if (urlSuffix && !url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [experiment_id,] = validateUrl(url, this.baseUrl, '/experiments/{}');
        const body = experimentUpdate;
        const parameters = {
            experiment_id: experiment_id,
            changedURL: options?.changedURL,
        };
        const query = [];
        if (parameters["changedURL"])
            query.push(["changedURL", parameters["changedURL"].toString()]);
        if (!RequestValidation.validateUpdateExperimentInput(parameters, body))
            throw new ValidationError('Request validation failed!', RequestValidation.validateUpdateExperimentInput.errors);
        const authorization = `Bearer ${this.accessToken}`;
        const response = await this.fetch(url.replace(this.baseUrl, this.serviceUrl) + '?' + new URLSearchParams(query), {
            method: "PATCH",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...this.fixedHeaders,
                ...(options?.headers ?? [])
            ],
            body: JSON.stringify(body)
        });
        if (!RequestValidation.validateUpdateExperimentOutput(response))
            throw new ValidationError('Response validation failed!', RequestValidation.validateUpdateExperimentOutput.errors);
        if (Types.isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Delete an experiment
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The experiment was deleted.
     */
    async deleteExperiment(url, options) {
        const urlSuffix = '/experiments/{}'.split('{}').at(-1) ?? '';
        if (urlSuffix && !url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [experiment_id,] = validateUrl(url, this.baseUrl, '/experiments/{}');
        const parameters = {
            experiment_id: experiment_id,
        };
        if (!RequestValidation.validateDeleteExperimentInput(parameters))
            throw new ValidationError('Request validation failed!', RequestValidation.validateDeleteExperimentInput.errors);
        const authorization = `Bearer ${this.accessToken}`;
        const response = await this.fetch(url.replace(this.baseUrl, this.serviceUrl), {
            method: "DELETE",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...this.fixedHeaders,
                ...(options?.headers ?? [])
            ]
        });
        if (!RequestValidation.validateDeleteExperimentOutput(response))
            throw new ValidationError('Response validation failed!', RequestValidation.validateDeleteExperimentOutput.errors);
        if (Types.isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    }
    /**
     * List templates
     *
     * @param options.url
     * Url of the  to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The list of all templates.
     */
    async listTemplate(options) {
        const url = appendToUrl(options?.url ?? this.baseUrl, "/templates");
        if (!RequestValidation.validateListTemplateInput())
            throw new ValidationError('Request validation failed!', RequestValidation.validateListTemplateInput.errors);
        const authorization = `Bearer ${this.accessToken}`;
        const response = await this.fetch(url.replace(this.baseUrl, this.serviceUrl), {
            method: "GET",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...this.fixedHeaders,
                ...(options?.headers ?? [])
            ]
        });
        if (!RequestValidation.validateListTemplateOutput(response))
            throw new ValidationError('Response validation failed!', RequestValidation.validateListTemplateOutput.errors);
        if (Types.isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Create a new template
     *
     * @param template
     * The template to be created.
     * @param options.url
     * Url of the  to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * 201: The template was created. A JSON representation of the new template is returned.
     * 202: The template was created and will be set to running eventually. A JSON representation of the new template is returned.
     */
    async createTemplate(template, options) {
        const url = appendToUrl(options?.url ?? this.baseUrl, "/templates");
        const body = template;
        if (!RequestValidation.validateCreateTemplateInput(body))
            throw new ValidationError('Request validation failed!', RequestValidation.validateCreateTemplateInput.errors);
        const authorization = `Bearer ${this.accessToken}`;
        const response = await this.fetch(url.replace(this.baseUrl, this.serviceUrl), {
            method: "POST",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...this.fixedHeaders,
                ...(options?.headers ?? [])
            ],
            body: JSON.stringify(body)
        });
        if (!RequestValidation.validateCreateTemplateOutput(response))
            throw new ValidationError('Response validation failed!', RequestValidation.validateCreateTemplateOutput.errors);
        if (Types.isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * View an template.
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON Representation of the template
     */
    async getTemplate(url, options) {
        const urlSuffix = '/templates/{}'.split('{}').at(-1) ?? '';
        if (urlSuffix && !url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [template_id,] = validateUrl(url, this.baseUrl, '/templates/{}');
        const parameters = {
            template_id: template_id,
        };
        if (!RequestValidation.validateGetTemplateInput(parameters))
            throw new ValidationError('Request validation failed!', RequestValidation.validateGetTemplateInput.errors);
        const authorization = `Bearer ${this.accessToken}`;
        const response = await this.fetch(url.replace(this.baseUrl, this.serviceUrl), {
            method: "GET",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...this.fixedHeaders,
                ...(options?.headers ?? [])
            ]
        });
        if (!RequestValidation.validateGetTemplateOutput(response))
            throw new ValidationError('Response validation failed!', RequestValidation.validateGetTemplateOutput.errors);
        if (Types.isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Update an existing template.
     *
     * @param url
     * Url of the resource to be accessed.
     * @param templateUpdate
     * Update the template
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * 200: The JSON Representation of the changed template
     * 202: The JSON Representation of the changed template, that will be set to running eventually
     */
    async updateTemplate(url, templateUpdate, options) {
        const urlSuffix = '/templates/{}'.split('{}').at(-1) ?? '';
        if (urlSuffix && !url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [template_id,] = validateUrl(url, this.baseUrl, '/templates/{}');
        const body = templateUpdate;
        const parameters = {
            template_id: template_id,
        };
        if (!RequestValidation.validateUpdateTemplateInput(parameters, body))
            throw new ValidationError('Request validation failed!', RequestValidation.validateUpdateTemplateInput.errors);
        const authorization = `Bearer ${this.accessToken}`;
        const response = await this.fetch(url.replace(this.baseUrl, this.serviceUrl), {
            method: "PATCH",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...this.fixedHeaders,
                ...(options?.headers ?? [])
            ],
            body: JSON.stringify(body)
        });
        if (!RequestValidation.validateUpdateTemplateOutput(response))
            throw new ValidationError('Response validation failed!', RequestValidation.validateUpdateTemplateOutput.errors);
        if (Types.isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Delete an template
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The template was deleted.
     */
    async deleteTemplate(url, options) {
        const urlSuffix = '/templates/{}'.split('{}').at(-1) ?? '';
        if (urlSuffix && !url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [template_id,] = validateUrl(url, this.baseUrl, '/templates/{}');
        const parameters = {
            template_id: template_id,
        };
        if (!RequestValidation.validateDeleteTemplateInput(parameters))
            throw new ValidationError('Request validation failed!', RequestValidation.validateDeleteTemplateInput.errors);
        const authorization = `Bearer ${this.accessToken}`;
        const response = await this.fetch(url.replace(this.baseUrl, this.serviceUrl), {
            method: "DELETE",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...this.fixedHeaders,
                ...(options?.headers ?? [])
            ]
        });
        if (!RequestValidation.validateDeleteTemplateOutput(response))
            throw new ValidationError('Response validation failed!', RequestValidation.validateDeleteTemplateOutput.errors);
        if (Types.isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    }
}
//# sourceMappingURL=client.js.map